import json

from flask import Blueprint, jsonify, Response, current_app

from inventory_provider.routes import common

routes = Blueprint("lg-query-routes", __name__)

ACCESS_PUBLIC = 'public'
ACCESS_INTERNAL = 'all'


@routes.after_request
def after_request(resp):
    return common.after_request(resp)


@routes.route("/routers/<string:access>", methods=['GET', 'POST'])
@common.require_accepts_json
def routers(access):

    if access not in {ACCESS_INTERNAL, ACCESS_PUBLIC}:
        return Response(
            response='unknown access level',
            status=404,
            mimetype='text/html')

    def _visible(router):
        if access == ACCESS_INTERNAL:
            return True
        return router['type'] == 'CORE'

    def _routers():
        for doc in common.load_json_docs(
                config_params=current_app.config['INVENTORY_PROVIDER_CONFIG'],
                key_pattern='opsdb:lg:*'):
            yield doc['value']

    redis = common.get_current_redis()
    cache_key = f'classifier-cache:lg:{access}'
    result = redis.get(cache_key)

    if result:
        result = json.loads(result.decode('utf-8'))
    else:
        result = list(filter(_visible, _routers()))
        # cache this data for the next call
        redis.set(cache_key, json.dumps(result).encode('utf-8'))

    if not result:
        return Response(
            response=f'no routers found for access level {access}',
            status=404,
            mimetype="text/html")

    return jsonify(result)
